<?php
/*
Plugin Name: FWP+: Limit size of posts
Plugin URI: http://projects.radgeek.com/fwp-limit-size-of-posts/
Description: enables you to limit the size of incoming syndicated posts from FeedWordPress by word count or by character count
Version: 2010.0120 
Author: Charles Johnson
Author URI: http://radgeek.com/
License: GPL
*/

$limiter = new FWPLimitSizeOfPosts;

add_filter(
	/*hook=*/ 'the_content',
	/*function=*/ array(&$limiter, 'the_content'),
	/*priority=*/ 10010,
	/*arguments=*/ 1
);

// We redo the excerpts, too, because it wouldn't do to have the excerpt
// being longer than the actual post!
add_filter(
	/*hook=*/ 'the_excerpt',
	/*function=*/ array(&$limiter, 'the_content'),
	/*priority=*/ 10010,
	/*arguments=*/ 1
);
add_filter(
	/*hook=*/ 'the_excerpt_rss',
	/*function=*/ array(&$limiter, 'the_content'),
	/*priority=*/ 10010,
	/*arguments=*/ 1
);

if (!function_exists('get_the_content_feed')) :
	// For WordPress < 2.9. This hook was deprecated in 2.9, and the_content
	// now should cover it.
	add_filter(
		/*hook=*/ 'the_content_rss',
		/*function=*/ array(&$limiter, 'the_content'),
		/*priority=*/ 10010,
		/*arguments=*/ 1
	);
endif;

add_action(
	/*hook=*/ 'feedwordpress_admin_page_posts_meta_boxes',
	/*function=*/ array(&$limiter, 'add_settings_box'),
	/*priority=*/ 100,
	/*arguments=*/ 1
);
add_action(
	/*hook=*/ 'feedwordpress_admin_page_posts_save',
	/*function=*/ array(&$limiter, 'save_settings'),
	/*priority=*/ 100,
	/*arguments=*/ 2
);

class FWPLimitSizeOfPosts {
	var $mb;
	var $myCharset;

	// Careful multibyte support (fallback to normal functions if not available)
	function FWPLimitSizeOfPosts () {
		$this->name = strtolower(get_class($this));
		$this->myCharset = get_bloginfo('charset');
		
		// Carefully support multibyte languages
		if (extension_loaded('mbstring') and function_exists('mb_list_encodings')) :
			$this->mb = in_array($this->myCharset, mb_list_encodings());
		endif;
	} /* FWPLimitSizeOfPosts constructor */
	
	function __construct () {
		self::FWPLimitSizeOfPosts();
	} /* FWPLimitSizeOfPosts::__construct() */

	// Carefully support multibyte languages
	function is_mb () { return $this->mb; }
	function charset () { return $this->myCharset; }
	
	function substr ($str, $start, $length = null) {
		$length = (is_null($length) ? $this->strlen($str) : $length);
		$str = ($this->is_mb()
			? mb_substr($str, $start, $length, $this->charset())
			: substr($str, $start, $length));
		return $str;
	} /* FWPLimitSizeOfPosts::substr() */
	
	function strlen ($str) {
		if ($this->is_mb()) :
			return mb_strlen($str, $this->charset());
		else :
			return strlen($str);
		endif;
	} /* FWPLimitSizeOfPosts::strlen() */

	function filter ($text, $params = array()) {
		global $id, $post;
		
		if (isset($params['words'])) : $wordsMax = $params['words']; else : $wordsMax = NULL; endif;
		if (isset($params['characters'])) : $charsMax = $params['characters']; else : $charsMax = NULL; endif;
		if (isset($params['ellipsis'])) : $ellipsis = $params['ellipsis'] ; else : $ellipsis = '...'; endif;
		if (isset($params['allowed tags'])) : $allowed_tags = $params['allowed tags']; else : $allowed_tags = NULL; endif;
		if (isset($params['finish word'])) : $finish_word = $params['finish word'] ; else : $finish_word = false; endif;

		$ellipsis = apply_filters('feedwordpress_limit_size_of_posts_ellipsis', $ellipsis);
			
		// From the default wp_trim_excerpt():
		// Some kind of precaution against malformed CDATA in RSS feeds I suppose
		$text = str_replace(']]>', ']]&gt;', $text);
		
		if (!is_null($allowed_tags)) :
			$text = strip_tags($text, $allowed_tags);
		endif;

		$wordsOK = (is_null($wordsMax) OR ($wordsMax > count(preg_split('/[\s]+/', strip_tags($text), -1))));
		$charsOK = (is_null($charsMax) OR ($charsMax > $this->strlen(strip_tags($text))));
		if ($wordsOK and $charsOK) :
			return $text;
		else :
			// Break string into "words" based on
			// (1) whitespace, or
			// (2) tags
			// Hence "un<em>frakking</em>believable" will
			// be treated as 3 words, not as 1 word. You
			// might refine this, if it is really important,
			// by keeping a list of "word-breaking" tags
			// (e.g. <br/>, <p>/</p>, <div>/</div>, etc.)
			// and non-word-breaking tags (e.g. <em>/</em>,
			// etc.).
			//
			// Tags do not count towards either words or characters;
			// Whitespace chunks count towards characters, not words

			$text_bits = preg_split('/([\s]+|<[^>]+>)/', $text, -1, PREG_SPLIT_DELIM_CAPTURE);

			$wordsN = 0;
			$charsN = 0;
			$length = 0;
			$text = '';
			foreach ($text_bits as $chunk) :
				// This is a tag, or whitespace.
				if (preg_match('/^<[^>]+>$/s', $chunk)) :
					// NOOP: tag
				elseif (strlen(trim($chunk)) == 0) :
					$charsN += $this->strlen($chunk);
				else :
					$charsN += $this->strlen($chunk);
					$wordsN += 1;
				endif;

				if (!is_null($wordsMax) and ($wordsN > $wordsMax)) :
					break;
				endif;
				
				$text .= $chunk;

				if (!is_null($charsMax) and ($charsN > $charsMax)) :
					$length += ($this->strlen($chunk) - ($charsN - $charsMax));
					break;
				else :
					$length += $this->strlen($chunk);
				endif;
			endforeach;

			if ($charsN > $charsMax and !$finish_word) :
				// Break it off right there!
				$text = $this->substr($text, 0, $length);
			endif;

			$wordsLimited = (!is_null($wordsMax) AND ($wordsN > $wordsMax));
			$charsLimited = (!is_null($charsMax) AND ($charsN > $charsMax));
			if ($wordsLimited OR $charsLimited) :
				$text = $text . $ellipsis;
			endif;

			$text = force_balance_tags($text);
		endif;
		return $text;
	} /* FWPLimitSizeOfPosts::filter() */

	function the_content ($content) {
		// Only affects syndicated posts
		if (is_syndicated()) :
			// Get a SyndicatedLink object for the source of this
			// syndicated post.
			$link_id = get_syndication_feed_id();
			$link = new SyndicatedLink($link_id);

			$rule = $link->setting('limit size of posts', $this->name.'_limit_size_of_posts', NULL);
			// Rules are stored in the format array('metric' => $count, 'metric' => $count);
			// 'metric' is the metric to be limited (characters|words)
			// $count is a numeric value indicating the maximum to limit it to
			// NULL indicates that no limiting rules have been stored.
			
			if (is_string($rule)) : $rule = unserialize($rule); endif;

			// Only operate in a feed context if we have permission
			$haveRule = is_array($rule);
			$feedChecks = (!is_feed() or (isset($rule['rss']) and $rule['rss']));
			if ($haveRule and $feedChecks) :
				$content = $this->filter($content, $rule);
			endif;
		endif;
		return $content;
	} /* FWPLimitSizeOfPosts::the_content() */

	function add_settings_box ($page) {
		add_meta_box(
			/*id=*/ "feedwordpress_{$this->name}_box",
			/*title=*/ __("Limit size of posts"),
			/*callback=*/ array(&$this, 'display_settings'),
			/*page=*/ $page->meta_box_context(),
			/*context=*/ $page->meta_box_context()
		);
	} /* FWPLimitSizeOfPosts::add_settings_box() */

	function display_settings ($page, $box = NULL) {
		$global_rule = get_option("feedwordpress_{$this->name}_limit_size_of_posts", NULL);
		if ($page->for_feed_settings()) :
			$rule = $page->link->setting('limit size of posts', NULL, NULL);
		else :
			$rule = $global_rule;
		endif;
		$thesePosts = $page->these_posts_phrase();	

		if (is_string($global_rule)) : $global_rule = unserialize($global_rule); endif;
	
		if (is_string($rule)) : $rule = unserialize($rule); endif;

		foreach (array('words', 'characters') as $thingy) :
			$checked[$thingy] = ((isset($rule[$thingy]) and !is_null($rule[$thingy])) ? 'checked="checked"' : '');
			$input[$thingy] = '<input type="text" size="4" name="'.$this->name.'_'.$thingy.'_limit" value="'.(isset($rule[$thingy]) ? $rule[$thingy] : 100).'" />';
		endforeach;
		$selector = array();
			
		$wordsSet = isset($rule['words']);
		$charsSet = isset($rule['characters']);
		$rssOn = (is_null($rule) or (isset($rule['rss']) and $rule['rss']));
			
		if (!($wordsSet or $charsSet)) : $selected = 'no';
		else : $selected = 'yes';
		endif;
		if ($page->for_feed_settings()) :
			if (is_null($rule)) : $selected = 'default'; endif;
		endif;
		
		if ($rssOn) : $checked['rss'] = ' checked="checked"';
		else : $checked['rss'] = '';
		endif;

		$selector[] = '<ul>';
		
		$magicId = $this->name.'-limit-controls';
		foreach (array('default', 'yes', 'no') as $response) :
			$boxId[$response] = $this->name.'-limit-'.$response;
			if ($response != 'yes') :
				$jsCall[$response] = "contextual_appearance('{$boxId[$response]}', null, /*disappear=*/ '{$magicId}', 'yes', 'table-row-group', true);";
			else :
				$jsCall[$response] = "contextual_appearance('{$boxId[$response]}', /*appear=*/ '{$magicId}', null, 'yes', 'table-row-group', true);";
			endif;
			$input[$response] = '<input type="radio" onclick="'.$jsCall[$response].'" id="'.$boxId[$response].'" name="'.$this->name.'_limits" value="'.$response.'" '.($selected==$response?' checked="checked"':'').' />';
		endforeach;

		if ($page->for_feed_settings()) :
			$siteWide = array();
			if (is_null($global_rule)) :
				$siteWide[] = 'no limit';
			else:
				if (isset($global_rule['words'])) : $siteWide[] = $global_rule['words'].' words'; endif;
				if (isset($global_rule['characters'])) : $siteWide[] = $global_rule['characters'].' characters'; endif;
				if (isset($global_rule['rss']) and $global_rule['rss']) : $siteWide[] = 'on homepage and in feed'; else : $siteWide[] = 'on homepage, but not in feed'; endif;
			endif;
			
			$selector[] = '<li><label>'.$input['default'].' '.sprintf(__("Use site-wide settings for $thesePosts (currently: %s)"), implode(' / ',$siteWide)).'</label></li>';
		endif;
		$selector[] = '<li><label>'.$input['no'].' '.__("Do not limit the size of $thesePosts").'</label></li>';
		$selector[] = '<li><label>'.$input['yes'].' '.__("Limit the size of $thesePosts").'</label></li>';
		$selector[] = '</ul>';
		?>
		<table class="form-table" cellspacing="2" cellpadding="5">
		<thead>
		<tr><th scope="row"><?php _e('Limit post size:'); ?></th>
		<td><?php print implode("\n", $selector); ?></td></tr>
		</thead>
		<tbody id="<?php print $this->name; ?>-limit-controls"<?php if ($selected!='yes') : ?> style="display: none"<?php endif; ?>>
		<tr><th scope="row"><?php _e('Word count:'); ?></th>
		<td><input type="checkbox" <?php print $checked['words']; ?> name="<?php print $this->name; ?>_limit_words" value="yes" /> <?php printf(__("Limit $thesePosts to no more than %s words"), $input['words']); ?></td>
		</tr>
		<tr><th scope="row"><?php _e('Characters:'); ?></th>
		<td><input type="checkbox" <?php print $checked['characters']; ?> name="<?php print $this->name; ?>_limit_characters" value="yes" /> <?php printf(__("Limit $thesePosts to no more than %s characters"), $input['characters']); ?></td>
		</tr>
		<tr><th scope="row"><?php _e('Feed:'); ?></th>
		<td><label><input type="checkbox" <?php print $checked['rss']; ?> name="<?php print $this->name; ?>_rss" value="yes" /> Limit the size of <?php print $thesePosts; ?> in my syndication feed as well as on my homepage.</label></td>
		</tr>
		</tbody>
		</table>
		<?php
	} /* FWPLimitSizeOfPosts::display_settings() */
	
	function save_settings ($params, $page) {
		if (isset($params['save']) or isset($params['submit'])) :
			if (isset($params[$this->name.'_limits'])) :
				$rule = array();
	
				if (isset($params[$this->name.'_rss']) and $params[$this->name.'_rss']=='yes') :
					$rule['rss'] = true;
				endif;
	
				switch ($params[$this->name.'_limits']) :
				case 'default' :
					$rule = NULL;
					break;
				case 'no' :
					// allows feeds to override a default limiting
					// policy with a no-limiting policy
					$rule = array ( 'limits' => 'none' );
					break;
				case 'yes' :
				default :
					foreach (array('words', 'characters') as $thingy) :
						if (isset($params[$this->name.'_limit_'.$thingy])
						and ($params[$this->name.'_limit_'.$thingy]=='yes')
						and (isset($params[$this->name.'_'.$thingy.'_limit']))) :
							$rule[$thingy] = (int) $params[$this->name.'_'.$thingy.'_limit'];
						endif;
					endforeach;
				endswitch;
				
				// Now let's write it.
				if ($page->for_feed_settings()) :
					if (!is_null($rule)) :
						$page->link->settings['limit size of posts'] = serialize($rule);
					else :
						unset($page->link->settings['limit size of posts']);
					endif;
					$page->link->save_settings(/*reload=*/ true);
				else :
					update_option("feedwordpress_{$this->name}_limit_size_of_posts", $rule);
				endif;
			endif;
		endif;
	} /* FWPLimitSizeOfPosts::save_settings() */

} /* class FWPLimitSizeOfPosts */
